package de.intevation.flys.exports;

import java.awt.Font;

import org.w3c.dom.Document;

import org.apache.log4j.Logger;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.Range;
import org.jfree.data.xy.XYSeries;

import de.intevation.artifactdatabase.state.ArtifactAndFacet;

import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.WQDay;
import de.intevation.flys.artifacts.resources.Resources;

import de.intevation.flys.jfree.FLYSAnnotation;
import de.intevation.flys.jfree.StyledXYSeries;


/**
 * An OutGenerator that generates duration curves.
 *
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class DurationCurveGenerator
extends      XYChartGenerator
implements   FacetTypes
{
    public static enum YAXIS {
        W(0),
        Q(1);
        public int idx;
        private YAXIS(int c) {
           idx = c;
        }
    }

    private static Logger logger =
        Logger.getLogger(DurationCurveGenerator.class);

    public static final String I18N_DURATION_W =
        "chart.duration.curve.curve.w";

    public static final String I18N_DURATION_Q =
        "chart.duration.curve.curve.q";

    public static final String I18N_CHART_TITLE =
        "chart.duration.curve.title";

    public static final String I18N_CHART_SUBTITLE =
        "chart.duration.curve.subtitle";

    public static final String I18N_XAXIS_LABEL =
        "chart.duration.curve.xaxis.label";

    public static final String I18N_YAXIS_LABEL =
        "chart.duration.curve.yaxis.label";

    public static final String I18N_CHART_TITLE_DEFAULT  =
        "Dauerlinie";

    public static final String I18N_XAXIS_LABEL_DEFAULT  =
        "Unterschreitungsdauer [Tage]";

    public static final String I18N_YAXIS_LABEL_DEFAULT  =
        "W [NN + m]";


    public DurationCurveGenerator() {
        super();
    }


    /**
     * Create Axis for given index.
     * @return axis with according internationalized label.
     */
    @Override
    protected NumberAxis createYAxis(int index) {
        Font labelFont = new Font("Tahoma", Font.BOLD, 14);
        String label   = getYAxisLabel(index);

        NumberAxis axis = createNumberAxis(index, label);
        if (index == YAXIS.W.idx) {
            axis.setAutoRangeIncludesZero(false);
        }
        axis.setLabelFont(labelFont);
        return axis;
    }


    @Override
    protected String getDefaultChartTitle() {
        return msg(I18N_CHART_TITLE, I18N_CHART_TITLE_DEFAULT);
    }


    @Override
    protected String getDefaultChartSubtitle() {
        double[] dist  = getRange();

        Object[] args = new Object[] {
            getRiverName(),
            dist[0]
        };

        return msg(I18N_CHART_SUBTITLE, "", args);
    }


    @Override
    protected void addSubtitles(JFreeChart chart) {
        String subtitle = getChartSubtitle();

        if (subtitle != null && subtitle.length() > 0) {
            chart.addSubtitle(new TextTitle(subtitle));
        }
    }


    @Override
    protected String getDefaultXAxisLabel() {
        return msg(I18N_XAXIS_LABEL, I18N_XAXIS_LABEL_DEFAULT);
    }


    @Override
    protected String getDefaultYAxisLabel(int index) {
        String label = "default";
        if (index == YAXIS.W.idx) {
            label = msg(I18N_YAXIS_LABEL, I18N_YAXIS_LABEL_DEFAULT);
        }
        else if (index == YAXIS.Q.idx) {
            // TODO i18n for this label
            label = "Q [m\u00b3/s]";
            //label = msg(get2YAxisLabelKey(), get2YAxisDefaultLabel());
        }

        return label;
    }


    @Override
    protected boolean zoomX(XYPlot plot, ValueAxis axis, Range range, Range x) {
        boolean zoomin = super.zoom(plot, axis, range, x);

        if (!zoomin) {
            axis.setLowerBound(0d);
        }

        axis.setUpperBound(364);

        return zoomin;
    }


    @Override
    public void doOut(
        ArtifactAndFacet artifactFacet,
        Document         attr,
        boolean          visible
    ) {
        String name = artifactFacet.getFacetName();

        logger.debug("DurationCurveGenerator.doOut: " + name);

        if (name == null || name.length() == 0) {
            logger.error("No facet given. Cannot create dataset.");
            return;
        }

        if (name.equals(DURATION_W)) {
            doWOut((WQDay) artifactFacet.getData(context), attr, visible);
        }
        else if (name.equals(DURATION_Q)) {
            doQOut((WQDay) artifactFacet.getData(context), attr, visible);
        }
        else if (name.equals(DURATION_MAINVALUES_Q)
                || name.equals(MAINVALUES_Q)
                || name.equals(COMPUTED_DISCHARGE_MAINVALUES_W)
                || name.equals(MAINVALUES_W)
        ) {
            doAnnotations(
                (FLYSAnnotation) artifactFacet.getData(context),
                artifactFacet.getFacet(), attr, visible);
        }
        else if (FacetTypes.IS.MANUALPOINTS(name)) {
            doPoints(artifactFacet.getData(context),
                artifactFacet.getFacetDescription(),
                attr, visible, YAXIS.W.idx);
        }
        else {
            logger.warn("Unknown facet name: " + name);
            return;
        }
    }


    /**
     * Creates the series for a duration curve's W facet.
     *
     * @param wqdays The WQDay store that contains the Ws.
     * @param theme
     */
    protected void doWOut(WQDay wqdays, Document theme, boolean visible) {
        logger.debug("DurationCurveGenerator.doWOut");

        // TODO find the correct series name
        XYSeries series = new StyledXYSeries(
            getSeriesName(getRiverName(), DURATION_W), theme);

        int size = wqdays.size();
        for (int i = 0; i < size; i++) {
            int  day = wqdays.getDay(i);
            double w = wqdays.getW(i);

            series.add((double) day, w);
        }

        addAxisSeries(series, YAXIS.W.idx, visible);
    }


    /**
     * Creates the series for a duration curve's Q facet.
     *
     * @param wqdays The WQDay store that contains the Qs.
     * @param theme
     */
    protected void doQOut(WQDay wqdays, Document theme, boolean visible) {
        logger.debug("DurationCurveGenerator.doQOut");

        // TODO find the correct series name
        XYSeries series = new StyledXYSeries(
            getSeriesName(getRiverName(), DURATION_Q), theme);

        int size = wqdays.size();
        for (int i = 0; i < size; i++) {
            int  day = wqdays.getDay(i);
            double q = wqdays.getQ(i);

            series.add((double) day, q);
        }

        addAxisSeries(series, YAXIS.Q.idx, visible);
    }


    protected String getSeriesName(String river, String type) {
        Object[] args = new Object[] { river };

        if (type == null || type.length() == 0) {
            logger.warn("No duration curve type given.");
            return "n/a";
        }
        else if (type.equals(DURATION_W)) {
            return Resources.getMsg(
                context.getMeta(),
                I18N_DURATION_W,
                "W",
                args);
        }
        else if (type.equals(DURATION_Q)) {
            return Resources.getMsg(
                context.getMeta(),
                I18N_DURATION_Q,
                "W",
                args);
        }

        logger.warn("Could not determine chart curve type: " + type);
        return type;
    }


    @Override
    protected YAxisWalker getYAxisWalker() {
        return new YAxisWalker() {
            @Override
            public int length() {
                return YAXIS.values().length;
            }

            @Override
            public String getId(int idx) {
                YAXIS[] yaxes = YAXIS.values();
                return yaxes[idx].toString();
            }
        };
    }

    // MainValue-Annotations should be visualized by a line that goes to the curve itself.
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
